#!/usr/bin/env python

import ssl
import socket
import logging
from datetime import datetime


########################################################################################################################


def set_timestamp(fmt, s):
    """Returns a date object of a string in the provided format (fmt).

    The string has to be in the correct format, if not None is returned."""

    try:
        ts = datetime.strptime(str(s.strip()), fmt)
    except ValueError:
        logging.error(f"Unable to convert provided argument '{str(s)}' to timestamp object")
        return

    return ts


class CertReport(object):
    def __init__(self, hostnames):
        self.hostnames = hostnames
        self.date_format = "%b %d %H:%M:%S %Y"
        self.now = datetime.now()
        self.slack_report = []
        self.certs = []

    def parse_certs(self):
        for c in self.hostnames:
            self.ssl_cert_expire(c)

    def ssl_cert_expire(self, crt):
        c = {"name": crt}
        context = ssl.create_default_context()

        with socket.create_connection((crt, 443)) as sock:
            try:
                with context.wrap_socket(sock, server_hostname=crt) as s:
                    s.do_handshake()
                    cert = s.getpeercert()
                    not_after = cert.get("notAfter")
                    ts = set_timestamp(self.date_format, not_after.rstrip("GMT"))
                    age = (self.now - ts).days
                    c["notAfter"] = not_after
                    c["expire_ts"] = ts
                    c["expire_age"] = age
            except ssl.SSLCertVerificationError as e:
                c["error_message"] = e.verify_message
                c["expire_age"] = 999999
            finally:
                self.certs.append(c)

            return

    def get_certs(self):
        return self.certs



#!/usr/bin/env python


########################################################################################################################


class SlackReport(object):
    def __init__(self, certs, warning, critical,
                 ok_symbol=":white_check_mark:",
                 error_symbol=":no_entry:",
                 warning_symbol=":warning:",
                 critical_symbol=":bangbang:",
                 expired_symbol=":rotating_light:",
                 skip_ok=False, title="SSL certificates report"):
        self.certs = certs
        self.warning = warning
        self.critical = critical
        self.skip_ok = skip_ok
        self.title = title
        self.ok_symbol = ok_symbol
        self.error_symbol = error_symbol
        self.warning_symbol = warning_symbol
        self.critical_symbol = critical_symbol
        self.expired_symbol = expired_symbol
        self.report = []

    def gen_report(self):
        for c in self.certs:
            name = c.get("name")
            expire_date = c.get("notAfter")
            days = c.get("expire_age")
            error_message = c.get("error_message")
            if error_message:
                row = f"{self.error_symbol} *{name}* - {error_message}"

            elif not error_message and isinstance(days, int) and days < 0:
                row = f"*{name}* - Will expire in {abs(days)} days ({expire_date})."
                if abs(days) <= self.critical:
                    row = f"{self.critical_symbol} {row}"
                elif self.warning >= abs(days) > self.critical:
                    row = f"{self.warning_symbol} {row}"
                elif self.skip_ok:
                    continue
                else:
                    row = f"{self.ok_symbol} {row}"

            elif not error_message and isinstance(days, int) and days >= 0:
                row = f"Cert **{name}** has already expired. Expired {abs(days)} days ago ({expire_date})."
                if abs(days) <= self.critical:
                    row = f"{self.expired_symbol} {row}"
            else:
                row = f"Unknown state for cert **{name}**."

            if row:
                row += "\n"
                self.report.append((days, row))

    def get_report_payload(self):
        rows = ""

        for e in sorted(self.report, reverse=True):
            rows += f"{e[-1]}\n"

        if rows:
            return {"text": f"*{self.title}*\n{rows}"}
